/*
 * main.c
 *
 * Copyright (C) 2004-2006 Atmark Techno, Inc.  All Rights Reserved.
 *
 */

#include <ctype.h>
#include <xuartlite_l.h>
#include "version.h"
#include "memmap.h"
#include "srec.h"
#include "medium.h"
#include "spi.h"
#include "flash.h"

#define LED_GPIO(v)        (*(volatile unsigned long *)(LED_REGISTER_BASEADDR) = (v))
#define LED_ON             (0)
#define LED_OFF            (1)

#define MAX_BUFFER_SIZE    (128)

#ifdef XPAR_SPI_FLASH_BASEADDR
#define BOOTLOADER_OFFSET_SPI        (0x00100000)
#else
#define FLASH_4MiB (0x16)
#define FLASH_8MiB (0x17)
#define BOOTLOADER_OFFSET_4MiB_FLASH (0x00080000)
#define BOOTLOADER_OFFSET_8MiB_FLASH (0x00100000)
#endif

static void busy_wait(int time)
{
        int i;
#ifdef __MICROBLAZE__
        time *= 6;
#endif
        for (i = 0; i < time; i++) {}
}

static int download(void)
{
        char buffer[MAX_BUFFER_SIZE];
        char *ptr1, *ptr2;

        while (1) {
                get_line(buffer, MAX_BUFFER_SIZE);

                /* Chew through any leading white space */
                ptr1 = buffer;
                while (*ptr1 && isspace(*ptr1))
                        ptr1++;

                /* And truncate any trailing white space */
                ptr2 = ptr1;
                while (*ptr2 && !isspace(*ptr2))
                        ptr2++;
                *ptr2 = '\0';

#ifdef XPAR_SPI_FLASH_BASEADDR
                put_char('.');
                /* check finish code */
                if (srec_decode_line(ptr1) == -1)
                        break;
#else
                srec_decode_line(ptr1);
                put_char('.');
#endif
        }

        return 0;
}

static unsigned int is_autoboot_mode(void)
{
#ifdef XPAR_SPI_FLASH_BASEADDR
        *(volatile unsigned int *)(SYSTEM_REGISTER_BASEADDR) = SYSTEM_REGISTER_BOOT_MODE;
#endif
        return *(volatile unsigned int *)(SYSTEM_REGISTER_BASEADDR) & SYSTEM_REGISTER_BOOT_MODE;
}

static unsigned int is_compatible_mode(void)
{
        return *(volatile unsigned int *)(SYSTEM_REGISTER_BASEADDR) & SYSTEM_REGISTER_COMPATIBLE_MODE;
}

static int get_bootloader_offset(unsigned int *offset)
{
        int ret = 0;
#ifdef XPAR_SPI_FLASH_BASEADDR
        *offset = BOOTLOADER_OFFSET_SPI;
#else
        switch (flash_get_size()) {
        case FLASH_4MiB:
                *offset = BOOTLOADER_OFFSET_4MiB_FLASH;
                break;
        case FLASH_8MiB:
                if (is_compatible_mode())
                        *offset = BOOTLOADER_OFFSET_4MiB_FLASH;
                else
                        *offset = BOOTLOADER_OFFSET_8MiB_FLASH;
                break;
        default:
                ret = -1;
                break;
        }
#endif
        return ret;
}

static void second_bootloader(unsigned int offset)
{
#ifdef XPAR_SPI_FLASH_BASEADDR
        spi_copy_to_dram(offset, (void *)(BBOOT_LOAD_ADDRESS), BOOTLOADER_SIZE);
#else
        flash_copy_to_dram(offset);
#endif
        jump_to_second_bootloader();    /* Never returns */
}

static void clear_rx_fifo(void)
{
        XUartLite_mSetControlReg(XPAR_CONSOLE_UART_BASEADDR, XUL_CR_FIFO_RX_RESET);
        XUartLite_mSetControlReg(XPAR_CONSOLE_UART_BASEADDR, 0);
}

int main(void)
{
        unsigned int bootloader_offset;
        char    key;

        LED_GPIO(LED_OFF);

#ifdef __PPC__
        XCache_DisableDCache();
        XCache_DisableICache();
        XCache_InvalidateICache();
        busy_wait(1000000);
#endif

        if (get_bootloader_offset(&bootloader_offset) < 0)
                goto halt;

        if (is_autoboot_mode())
                second_bootloader(bootloader_offset);

        myprint("\r\n\r\n"BBOOT_NAME " v" BBOOT_VERSION " (" TARGET_CPU ")\r\n");
        myprint("Press 'z' or 'Z' for BBoot Menu.\r\n");

        /* busy loop to wait getting a char 'z' or 'Z' */
        busy_wait(150000000);
        if (XUartLite_mIsReceiveEmpty(XPAR_CONSOLE_UART_BASEADDR) ||
            ((key = get_char()) != 0 && key != 'z' && key != 'Z'))
                second_bootloader(bootloader_offset);

        /* clear for long time pushing */
        clear_rx_fifo();

        myprint("\r\n\r\nPlease choose one of the following and hit enter.\r\n");
        myprint("a: activate second stage bootloader (default)\r\n");
        myprint("s: download a s-record file\r\n");

        while (1) {
                key = get_char();
                switch (key) {
                case 'a':    /* activate second stage bootloader */
                case 'A':
                case '\r':
                case '\n':
                        second_bootloader(bootloader_offset);
                        break;
                case 's':
                case 'S':
#ifndef XPAR_SPI_FLASH_BASEADDR
                        myprint("Erasing Flash...");
                        flash_erase(bootloader_offset);
                        myprint("Done\r\n");
#endif
                        myprint("Start sending S-Record!!\r\n");
                        download();
                        break;
                default:
                        myprint("Invalid selection.\r\n");
                case 'z':
                case 'Z':
                        clear_rx_fifo();
                        break;
                }
        }
halt:
        myprint("Halting...\r\n");
        return 0;
}
